@@ -5,49 +5,26 @@ This module defines utilities for internal use. This is not a public interface
 and may be changed without notice.
 """
 
-from __future__ import absolute_import
-
 import datetime
 import glob
 import numbers
 import os
 import platform
 import re
-import sys
 import warnings
-
-try:
-    from collections.abc import Iterable
-except ImportError:
-    from collections import Iterable
-try:
-    from urllib.parse import unquote, urlparse
-except ImportError:
-    from urlparse import urlparse, unquote
+from collections.abc import Iterable
+from urllib.parse import unquote, urlparse
 
 import numpy
+import packaging
 
-py2 = sys.version_info[0] <= 2
-py26 = py2 and sys.version_info[1] <= 6
-py27 = py2 and not py26
-py35 = not py2 and sys.version_info[1] <= 5
-py36 = not py2 and sys.version_info[1] <= 6
 win = platform.system().lower().startswith("win")
 
 
-if py2:
-    # to silence flake8 F821 errors
-    unicode = eval("unicode")
-    range = eval("xrange")
-else:
-    unicode = None
-    range = eval("range")
-
-
 def tobytes(array):
     """
     Calls ``array.tobytes()`` or its older equivalent, ``array.tostring()``,
-    depending on what's available in this NumPy version.
+    depending on what's available in this NumPy version. (tobytes added in 1.9)
     """
     if hasattr(array, "tobytes"):
         return array.tobytes()
@@ -79,24 +56,19 @@ def isstr(x):
     """
     Returns True if and only if ``x`` is a string (including Python 2 unicode).
     """
-    if py2:
-        return isinstance(x, (bytes, unicode))
-    else:
-        return isinstance(x, str)
+    return isinstance(x, str)
 
 
 def ensure_str(x):
     """
     Ensures that ``x`` is a string (decoding with 'surrogateescape' if necessary).
     """
-    if not py2 and isinstance(x, bytes):
+    if isinstance(x, bytes):
         return x.decode(errors="surrogateescape")
-    elif py2 and isinstance(x, unicode):
-        return x.encode()
     elif isinstance(x, str):
         return x
     else:
-        raise TypeError("expected a string, not {0}".format(type(x)))
+        raise TypeError(f"expected a string, not {type(x)}")
 
 
 def ensure_numpy(array, types=(numpy.bool_, numpy.integer, numpy.floating)):
@@ -111,13 +83,40 @@ def ensure_numpy(array, types=(numpy.bool_, numpy.integer, numpy.floating)):
         except (ValueError, numpy.VisibleDeprecationWarning):
             raise TypeError("cannot be converted to a NumPy array")
         if not issubclass(out.dtype.type, types):
-            raise TypeError(
-                "cannot be converted to a NumPy array of type {0}".format(types)
-            )
+            raise TypeError(f"cannot be converted to a NumPy array of type {types}")
         return out
 
 
-_regularize_filter_regex = re.compile("^/(.*)/([iLmsux]*)$")
+def parse_version(version):
+    """
+    Converts a semver string into a Version object that can be compared with
+    ``<``, ``>=``, etc.
+
+    Currently implemented using ``packaging.version.parse``
+    (exposing that library in the return type).
+    """
+    return packaging.version.parse(version)
+
+
+def from_module(obj, module_name):
+    """
+    Returns True if ``obj`` is an instance of a class from a module
+    given by name.
+
+    This is like ``isinstance`` (in that it searches the whole ``mro``),
+    except that the module providing the type to check against doesn't
+    have to be imported and doesn't get imported (as a side effect) by
+    this function.
+    """
+    try:
+        mro = type(obj).mro()
+    except TypeError:
+        return False
+
+    for t in mro:
+        if t.__module__ == module_name or t.__module__.startswith(module_name + "."):
+            return True
+    return False
 
 
 def _regularize_filter_regex_flags(flags):
@@ -145,6 +144,9 @@ def no_filter(x):
     return True
 
 
+_regularize_filter_regex = re.compile("^/(.*)/([iLmsux]*)$")
+
+
 def regularize_filter(filter):
     """
     Convert None, str, iterable of str, wildcards, and regular expressions into
@@ -170,7 +172,7 @@ def regularize_filter(filter):
     else:
         raise TypeError(
             "filter must be None, callable, a regex string between slashes, or a "
-            "glob pattern, not {0}".format(repr(filter))
+            "glob pattern, not {}".format(repr(filter))
         )
 
 
@@ -239,7 +241,7 @@ def regularize_rename(rename):
 
     raise TypeError(
         "rename must be None, callable, a '/from/to/' regex, an iterable of "
-        "regex rules, or a dict, not {0}".format(repr(rename))
+        "regex rules, or a dict, not {}".format(repr(rename))
     )
 
 
@@ -380,7 +382,7 @@ def file_path_to_source_class(file_path, options):
         return out, file_path
 
     else:
-        raise ValueError("URI scheme not recognized: {0}".format(file_path))
+        raise ValueError(f"URI scheme not recognized: {file_path}")
 
 
 if isinstance(__builtins__, dict):
@@ -445,35 +447,35 @@ def memory_size(data, error_message=None):
             if unit == "KB":
                 target *= 1000
             elif unit == "MB":
-                target *= 1000 ** 2
+                target *= 1000**2
             elif unit == "GB":
-                target *= 1000 ** 3
+                target *= 1000**3
             elif unit == "TB":
-                target *= 1000 ** 4
+                target *= 1000**4
             elif unit == "PB":
-                target *= 1000 ** 5
+                target *= 1000**5
             elif unit == "EB":
-                target *= 1000 ** 6
+                target *= 1000**6
             elif unit == "ZB":
-                target *= 1000 ** 7
+                target *= 1000**7
             elif unit == "YB":
-                target *= 1000 ** 8
+                target *= 1000**8
             elif unit == "KIB":
                 target *= 1024
             elif unit == "MIB":
-                target *= 1024 ** 2
+                target *= 1024**2
             elif unit == "GIB":
-                target *= 1024 ** 3
+                target *= 1024**3
             elif unit == "TIB":
-                target *= 1024 ** 4
+                target *= 1024**4
             elif unit == "PIB":
-                target *= 1024 ** 5
+                target *= 1024**5
             elif unit == "EIB":
-                target *= 1024 ** 6
+                target *= 1024**6
             elif unit == "ZIB":
-                target *= 1024 ** 7
+                target *= 1024**7
             elif unit == "YIB":
-                target *= 1024 ** 8
+                target *= 1024**8
             return int(target)
 
     if isint(data):
@@ -482,7 +484,7 @@ def memory_size(data, error_message=None):
     if error_message is None:
         raise TypeError(
             "number of bytes or memory size string with units "
-            "(such as '100 MB') required, not {0}".format(repr(data))
+            f"(such as '100 MB') required, not {data!r}"
         )
     else:
         raise TypeError(error_message)
@@ -545,7 +547,7 @@ def awkward_form(
                     '"float64"'
                 )
             else:
-                raise AssertionError("{0}: {1}".format(repr(model), type(model)))
+                raise AssertionError(f"{model!r}: {type(model)}")
 
         return _primitive_awkward_form[model]
 
@@ -622,11 +624,11 @@ def awkward_form_remove_uproot(awkward, form):
         )
     elif isinstance(form, awkward.forms.RecordForm):
         return awkward.forms.RecordForm(
-            dict(
-                (k, awkward_form_remove_uproot(awkward, v))
+            {
+                k: awkward_form_remove_uproot(awkward, v)
                 for k, v in form.contents.items()
                 if not k.startswith("@")
-            ),
+            },
             form.has_identities,
             parameters,
         )
@@ -659,7 +661,7 @@ def awkward_form_remove_uproot(awkward, form):
             parameters,
         )
     else:
-        raise RuntimeError("unrecognized form: {0}".format(type(form)))
+        raise RuntimeError(f"unrecognized form: {type(form)}")
 
 
 # FIXME: Until we get Awkward reading these bytes directly, rather than
@@ -770,9 +772,7 @@ def awkward_form_of_iter(awkward, form):
         return out
     elif isinstance(form, awkward.forms.RecordForm):
         return awkward.forms.RecordForm(
-            dict(
-                (k, awkward_form_of_iter(awkward, v)) for k, v in form.contents.items()
-            ),
+            {k: awkward_form_of_iter(awkward, v) for k, v in form.contents.items()},
             form.has_identities,
             form.parameters,
         )
@@ -805,7 +805,7 @@ def awkward_form_of_iter(awkward, form):
             form.parameters,
         )
     else:
-        raise RuntimeError("unrecognized form: {0}".format(type(form)))
+        raise RuntimeError(f"unrecognized form: {type(form)}")
 
 
 def damerau_levenshtein(a, b, ratio=False):
@@ -869,7 +869,7 @@ def code_to_datetime(code):
         ((code & 0b00000000001111100000000000000000) >> 17),
         ((code & 0b00000000000000011111000000000000) >> 12),
         ((code & 0b00000000000000000000111111000000) >> 6),
-        ((code & 0b00000000000000000000000000111111)),
+        (code & 0b00000000000000000000000000111111),
     )
 
 
