@@ -22,7 +22,6 @@ Lazy arrays (:doc:`uproot.behaviors.TBranch.lazy`) can only use the
 :doc:`uproot.interpretation.library.Awkward` library.
 """
 
-from __future__ import absolute_import
 
 import gc
 import itertools
@@ -40,7 +39,7 @@ def _rename(name, context):
         return context["rename"]
 
 
-class Library(object):
+class Library:
     """
     Abstract superclass of array-library handlers, for libraries such as NumPy,
     Awkward Array, and Pandas.
@@ -108,7 +107,7 @@ class Library(object):
         raise AssertionError
 
     def group(self, arrays, expression_context, how):
-        u"""
+        """
         Args:
             arrays (dict of str \u2192 array): Mapping from names to finalized
                 array objets to combine into a group.
@@ -127,12 +126,10 @@ class Library(object):
         elif how is list:
             return [arrays[name] for name, _ in expression_context]
         elif how is dict or how is None:
-            return dict(
-                (_rename(name, c), arrays[name]) for name, c in expression_context
-            )
+            return {_rename(name, c): arrays[name] for name, c in expression_context}
         else:
             raise TypeError(
-                "for library {0}, how must be tuple, list, dict, or None (for "
+                "for library {}, how must be tuple, list, dict, or None (for "
                 "dict)".format(self.name)
             )
 
@@ -170,7 +167,7 @@ class Library(object):
 
 
 class NumPy(Library):
-    u"""
+    """
     A :doc:`uproot.interpretation.library.Library` that presents ``TBranch``
     data as NumPy arrays. The standard name for this library is ``"np"``.
 
@@ -240,18 +237,18 @@ class NumPy(Library):
             return all_arrays
 
         if isinstance(all_arrays[0], (tuple, list)):
-            keys = uproot._util.range(len(all_arrays[0]))
+            keys = range(len(all_arrays[0]))
         elif isinstance(all_arrays[0], dict):
             keys = list(all_arrays[0])
         else:
             raise AssertionError(repr(all_arrays[0]))
 
-        to_concatenate = dict((k, []) for k in keys)
+        to_concatenate = {k: [] for k in keys}
         for arrays in all_arrays:
             for k in keys:
                 to_concatenate[k].append(arrays[k])
 
-        concatenated = dict((k, numpy.concatenate(to_concatenate[k])) for k in keys)
+        concatenated = {k: numpy.concatenate(to_concatenate[k]) for k in keys}
 
         if isinstance(all_arrays[0], tuple):
             return tuple(concatenated[k] for k in keys)
@@ -461,7 +458,7 @@ def _awkward_json_to_array(awkward, form, array):
 
 
 class Awkward(Library):
-    u"""
+    """
     A :doc:`uproot.interpretation.library.Library` that presents ``TBranch``
     data as Awkward Arrays. The standard name for this library is ``"ak"``.
 
@@ -563,14 +560,14 @@ class Awkward(Library):
                 )
             except uproot.interpretation.objects.CannotBeAwkward as err:
                 raise ValueError(
-                    """cannot produce Awkward Arrays for interpretation {0} because
+                    """cannot produce Awkward Arrays for interpretation {} because
 
-    {1}
+    {}
 
 instead, try library="np" instead of library="ak" or globally set uproot.default_library
 
-in file {2}
-in object {3}""".format(
+in file {}
+in object {}""".format(
                         repr(interpretation),
                         err.because,
                         interpretation.branch.file.file_path,
@@ -611,18 +608,13 @@ in object {3}""".format(
         elif how is list:
             return [arrays[name] for name, _ in expression_context]
         elif how is dict:
-            return dict(
-                (_rename(name, c), arrays[name]) for name, c in expression_context
-            )
+            return {_rename(name, c): arrays[name] for name, c in expression_context}
         elif how is None:
             if len(expression_context) == 0:
                 return awkward.Array(awkward.layout.RecordArray([], keys=[]))
             else:
                 return awkward.Array(
-                    dict(
-                        (_rename(name, c), arrays[name])
-                        for name, c in expression_context
-                    )
+                    {_rename(name, c): arrays[name] for name, c in expression_context}
                 )
         elif how == "zip":
             nonjagged = []
@@ -666,7 +658,7 @@ in object {3}""".format(
                     out = awkward.Array(awkward.layout.RecordArray([], keys=[]))
                 else:
                     out = awkward.Array(
-                        dict((name, renamed_arrays[name]) for name in nonjagged),
+                        {name: renamed_arrays[name] for name in nonjagged},
                     )
             for number, jagged in enumerate(jaggeds):
                 cut = len(jagged[0])
@@ -686,14 +678,14 @@ in object {3}""".format(
                 ):
                     cut = 0
                 if cut == 0:
-                    common = "jagged{0}".format(number)
+                    common = f"jagged{number}"
                     if len(jagged) == 0:
                         subarray = awkward.Array(
                             awkward.layout.RecordArray([], keys=[])
                         )
                     else:
                         subarray = awkward.zip(
-                            dict((name, renamed_arrays[name]) for name in jagged)
+                            {name: renamed_arrays[name] for name in jagged}
                         )
                 else:
                     common = jagged[0][:cut].strip("_./")
@@ -703,10 +695,10 @@ in object {3}""".format(
                         )
                     else:
                         subarray = awkward.zip(
-                            dict(
-                                (name[cut:].strip("_./"), renamed_arrays[name])
+                            {
+                                name[cut:].strip("_./"): renamed_arrays[name]
                                 for name in jagged
-                            )
+                            }
                         )
                 if out is None:
                     out = awkward.Array({common: subarray})
@@ -716,7 +708,7 @@ in object {3}""".format(
             return out
         else:
             raise TypeError(
-                'for library {0}, how must be tuple, list, dict, "zip" for '
+                'for library {}, how must be tuple, list, dict, "zip" for '
                 "a record array with jagged arrays zipped, if possible, or "
                 "None, for an unzipped record array".format(self.name)
             )
@@ -728,18 +720,18 @@ in object {3}""".format(
             return all_arrays
 
         if isinstance(all_arrays[0], (tuple, list)):
-            keys = uproot._util.range(len(all_arrays[0]))
+            keys = range(len(all_arrays[0]))
         elif isinstance(all_arrays[0], dict):
             keys = list(all_arrays[0])
         else:
             return awkward.concatenate(all_arrays)
 
-        to_concatenate = dict((k, []) for k in keys)
+        to_concatenate = {k: [] for k in keys}
         for arrays in all_arrays:
             for k in keys:
                 to_concatenate[k].append(arrays[k])
 
-        concatenated = dict((k, awkward.concatenate(to_concatenate[k])) for k in keys)
+        concatenated = {k: awkward.concatenate(to_concatenate[k]) for k in keys}
 
         if isinstance(all_arrays[0], tuple):
             return tuple(concatenated[k] for k in keys)
@@ -749,10 +741,17 @@ in object {3}""".format(
             return concatenated
 
 
-def _pandas_rangeindex():
-    import pandas
+def _is_pandas_rangeindex(pandas, index):
+    if hasattr(pandas, "RangeIndex") and isinstance(index, pandas.RangeIndex):
+        return True
+    if hasattr(index, "is_integer") and index.is_integer():
+        return True
+    if uproot._util.parse_version(pandas.__version__) < uproot._util.parse_version(
+        "1.4.0"
+    ) and isinstance(index, pandas.Int64Index):
+        return True
 
-    return (getattr(pandas, "RangeIndex", pandas.Int64Index), pandas.Int64Index)
+    return False
 
 
 def _strided_to_pandas(path, interpretation, data, arrays, columns):
@@ -770,7 +769,7 @@ def _pandas_basic_index(pandas, entry_start, entry_stop):
     if hasattr(pandas, "RangeIndex"):
         return pandas.RangeIndex(entry_start, entry_stop)
     else:
-        return pandas.Int64Index(uproot._util.range(entry_start, entry_stop))
+        return pandas.Int64Index(range(entry_start, entry_stop))
 
 
 def _pandas_only_series(pandas, original_arrays, expression_context):
@@ -812,7 +811,7 @@ def _pandas_memory_efficient(pandas, series, names):
 
 
 class Pandas(Library):
-    u"""
+    """
     A :doc:`uproot.interpretation.library.Library` that presents ``TBranch``
     data as Pandas Series and DataFrames. The standard name for this library is
     ``"pd"``.
@@ -911,9 +910,7 @@ class Pandas(Library):
             names = []
             arrays = {}
             for n in array.dtype.names:
-                for tup in itertools.product(
-                    *[uproot._util.range(d) for d in array.shape[1:]]
-                ):
+                for tup in itertools.product(*[range(d) for d in array.shape[1:]]):
                     name = (n + "".join("[" + str(x) + "]" for x in tup),)
                     names.append(name)
                     arrays[name] = array[n][(slice(None),) + tup]
@@ -924,7 +921,7 @@ class Pandas(Library):
 
         elif array.dtype.names is not None:
             columns = pandas.MultiIndex.from_tuples([(x,) for x in array.dtype.names])
-            arrays = dict((y, array[x]) for x, y in zip(array.dtype.names, columns))
+            arrays = {y: array[x] for x, y in zip(array.dtype.names, columns)}
             index = _pandas_basic_index(pandas, entry_start, entry_stop)
             out = pandas.DataFrame(arrays, columns=columns, index=index)
             out.leaflist = True
@@ -933,9 +930,7 @@ class Pandas(Library):
         elif len(array.shape) != 1:
             names = []
             arrays = {}
-            for tup in itertools.product(
-                *[uproot._util.range(d) for d in array.shape[1:]]
-            ):
+            for tup in itertools.product(*[range(d) for d in array.shape[1:]]):
                 name = "".join("[" + str(x) + "]" for x in tup)
                 names.append(name)
                 arrays[name] = array[(slice(None),) + tup]
@@ -958,9 +953,7 @@ class Pandas(Library):
             return [arrays[name] for name, _ in expression_context]
 
         elif how is dict:
-            return dict(
-                (_rename(name, c), arrays[name]) for name, c in expression_context
-            )
+            return {_rename(name, c): arrays[name] for name, c in expression_context}
 
         elif uproot._util.isstr(how) or how is None:
             arrays, names = _pandas_only_series(pandas, arrays, expression_context)
@@ -978,7 +971,7 @@ class Pandas(Library):
                 arrays = newarrays
                 names = pandas.MultiIndex.from_tuples(newnames)
 
-            if all(isinstance(x.index, _pandas_rangeindex()) for x in arrays.values()):
+            if all(_is_pandas_rangeindex(pandas, x.index) for x in arrays.values()):
                 return _pandas_memory_efficient(pandas, arrays, names)
 
             indexes = []
@@ -1000,7 +993,7 @@ class Pandas(Library):
                 for index, group, df, gn in zip(indexes, groups, dfs, group_names):
                     for name in names:
                         array = arrays[name]
-                        if isinstance(array.index, _pandas_rangeindex()):
+                        if _is_pandas_rangeindex(pandas, array.index):
                             if flat_index is None or len(flat_index) != len(
                                 array.index
                             ):
@@ -1056,7 +1049,7 @@ class Pandas(Library):
             else:
                 out = None
                 for index, group in zip(indexes, groups):
-                    only = dict((name, arrays[name]) for name in group)
+                    only = {name: arrays[name] for name in group}
                     df = pandas.DataFrame(data=only, index=index, columns=group)
                     if out is None:
                         out = df
@@ -1067,16 +1060,16 @@ class Pandas(Library):
                 flat_names = [
                     name
                     for name in names
-                    if isinstance(arrays[name].index, _pandas_rangeindex())
+                    if _is_pandas_rangeindex(pandas, arrays[name].index)
                 ]
                 if len(flat_names) > 0:
                     flat_index = pandas.MultiIndex.from_arrays(
                         [arrays[flat_names[0]].index]
                     )
-                    only = dict(
-                        (name, pandas.Series(arrays[name].values, index=flat_index))
+                    only = {
+                        name: pandas.Series(arrays[name].values, index=flat_index)
                         for name in flat_names
-                    )
+                    }
                     df = pandas.DataFrame(
                         data=only, index=flat_index, columns=flat_names
                     )
@@ -1091,7 +1084,7 @@ class Pandas(Library):
 
         else:
             raise TypeError(
-                "for library {0}, how must be tuple, list, dict, str (for "
+                "for library {}, how must be tuple, list, dict, str (for "
                 "pandas.merge's 'how' parameter, or None (for one or more"
                 "DataFrames without merging)".format(self.name)
             )
@@ -1134,18 +1127,18 @@ class Pandas(Library):
             return all_arrays
 
         if isinstance(all_arrays[0], (tuple, list)):
-            keys = uproot._util.range(len(all_arrays[0]))
+            keys = range(len(all_arrays[0]))
         elif isinstance(all_arrays[0], dict):
             keys = list(all_arrays[0])
         else:
             return pandas.concat(all_arrays)
 
-        to_concatenate = dict((k, []) for k in keys)
+        to_concatenate = {k: [] for k in keys}
         for arrays in all_arrays:
             for k in keys:
                 to_concatenate[k].append(arrays[k])
 
-        concatenated = dict((k, pandas.concat(to_concatenate[k])) for k in keys)
+        concatenated = {k: pandas.concat(to_concatenate[k]) for k in keys}
 
         if isinstance(all_arrays[0], tuple):
             return tuple(concatenated[k] for k in keys)
@@ -1184,7 +1177,7 @@ def _regularize_library(library):
             return _libraries[library.name]
         else:
             raise ValueError(
-                "library {0} ({1}) cannot be used in this function".format(
+                "library {} ({}) cannot be used in this function".format(
                     type(library).__name__, repr(library.name)
                 )
             )
@@ -1194,7 +1187,7 @@ def _regularize_library(library):
             return _libraries[library().name]
         else:
             raise ValueError(
-                "library {0} ({1}) cannot be used in this function".format(
+                "library {} ({}) cannot be used in this function".format(
                     library.__name__, repr(library().name)
                 )
             )
@@ -1204,7 +1197,7 @@ def _regularize_library(library):
             return _libraries[library]
         except KeyError:
             raise ValueError(
-                """library {0} not recognized (for this function); """
+                """library {} not recognized (for this function); """
                 """try "np" (NumPy), "ak" (Awkward Array), or "pd" (Pandas) """
                 """instead""".format(repr(library))
             )
@@ -1226,7 +1219,7 @@ def _regularize_library_lazy(library):
             return _libraries_lazy[library.name]
         else:
             raise ValueError(
-                "library {0} ({1}) cannot be used in this function".format(
+                "library {} ({}) cannot be used in this function".format(
                     type(library).__name__, repr(library.name)
                 )
             )
@@ -1236,7 +1229,7 @@ def _regularize_library_lazy(library):
             return _libraries_lazy[library().name]
         else:
             raise ValueError(
-                "library {0} ({1}) cannot be used in this function".format(
+                "library {} ({}) cannot be used in this function".format(
                     library.__name__, repr(library().name)
                 )
             )
@@ -1246,7 +1239,7 @@ def _regularize_library_lazy(library):
             return _libraries_lazy[library]
         except KeyError:
             raise ValueError(
-                """library {0} not recognized (for this function); """
+                """library {} not recognized (for this function); """
                 """try "ak" (Awkward Array) """
                 """instead""".format(repr(library))
             )
