@@ -16,7 +16,6 @@ several numerical types:
   for some ``N``.
 """
 
-from __future__ import absolute_import
 
 import numpy
 
@@ -183,9 +182,9 @@ class AsDtype(Numerical):
 
     def __repr__(self):
         if self._to_dtype == self._from_dtype.newbyteorder("="):
-            return "AsDtype({0})".format(repr(str(self._from_dtype)))
+            return f"AsDtype({repr(str(self._from_dtype))})"
         else:
-            return "AsDtype({0}, {1})".format(
+            return "AsDtype({}, {})".format(
                 repr(str(self._from_dtype)), repr(str(self._to_dtype))
             )
 
@@ -271,7 +270,7 @@ class AsDtype(Numerical):
     def cache_key(self):
         def form(dtype, name):
             d, s = _dtype_shape(dtype)
-            return "{0}{1}{2}({3}{4})".format(
+            return "{}{}{}({}{})".format(
                 _numpy_byteorder_to_cache_key[d.byteorder],
                 d.kind,
                 d.itemsize,
@@ -302,7 +301,7 @@ class AsDtype(Numerical):
                 + "]"
             )
 
-        return "{0}({1},{2})".format(type(self).__name__, from_dtype, to_dtype)
+        return f"{type(self).__name__}({from_dtype},{to_dtype})"
 
     @property
     def typename(self):
@@ -318,8 +317,7 @@ class AsDtype(Numerical):
             return (
                 "struct {"
                 + " ".join(
-                    "{0} {1};".format(form(self.from_dtype[n]), n)
-                    for n in self.from_dtype.names
+                    f"{form(self.from_dtype[n])} {n};" for n in self.from_dtype.names
                 )
                 + "}"
             )
@@ -342,9 +340,9 @@ class AsDtype(Numerical):
             output = data.view(dtype).reshape((-1,) + shape)
         except ValueError:
             raise ValueError(
-                """basket {0} in tree/branch {1} has the wrong number of bytes ({2}) """
-                """for interpretation {3}
-in file {4}""".format(
+                """basket {} in tree/branch {} has the wrong number of bytes ({}) """
+                """for interpretation {}
+in file {}""".format(
                     basket.basket_num,
                     branch.object_path,
                     len(data),
@@ -505,8 +503,8 @@ class TruncatedNumerical(Numerical):
     def __repr__(self):
         args = [repr(self._low), repr(self._high), repr(self._num_bits)]
         if self._to_dims != ():
-            args.append("to_dims={0}".format(repr(self._to_dims)))
-        return "{0}({1})".format(type(self).__name__, ", ".join(args))
+            args.append(f"to_dims={self._to_dims!r}")
+        return "{}({})".format(type(self).__name__, ", ".join(args))
 
     def __eq__(self, other):
         return (
@@ -523,7 +521,7 @@ class TruncatedNumerical(Numerical):
 
     @property
     def cache_key(self):
-        return "{0}({1},{2},{3},{4})".format(
+        return "{}({},{},{},{})".format(
             type(self).__name__, self._low, self._high, self._num_bits, self._to_dims
         )
 
@@ -544,9 +542,9 @@ class TruncatedNumerical(Numerical):
             raw = data.view(self.from_dtype)
         except ValueError:
             raise ValueError(
-                """basket {0} in tree/branch {1} has the wrong number of bytes ({2}) """
-                """for interpretation {3} (expecting raw array of {4})
-in file {5}""".format(
+                """basket {} in tree/branch {} has the wrong number of bytes ({}) """
+                """for interpretation {} (expecting raw array of {})
+in file {}""".format(
                     basket.basket_num,
                     branch.object_path,
                     len(data),
@@ -617,9 +615,7 @@ class AsDouble32(TruncatedNumerical):
         if not uproot._util.isint(num_bits) or not 2 <= num_bits <= 32:
             raise TypeError("num_bits must be an integer between 2 and 32 (inclusive)")
         if high <= low and not self.is_truncated:
-            raise ValueError(
-                "high ({0}) must be strictly greater than low ({1})".format(high, low)
-            )
+            raise ValueError(f"high ({high}) must be strictly greater than low ({low})")
 
     @property
     def to_dtype(self):
@@ -685,9 +681,7 @@ class AsFloat16(TruncatedNumerical):
         if not uproot._util.isint(num_bits) or not 2 <= num_bits <= 32:
             raise TypeError("num_bits must be an integer between 2 and 32 (inclusive)")
         if high <= low and not self.is_truncated:
-            raise ValueError(
-                "high ({0}) must be strictly greater than low ({1})".format(high, low)
-            )
+            raise ValueError(f"high ({high}) must be strictly greater than low ({low})")
 
     @property
     def to_dtype(self):
