@@ -14,17 +14,12 @@ and sometimes freeing data.
 See :doc:`uproot.writing._cascade` for a general overview of the cascading writer concept.
 """
 
-from __future__ import absolute_import
 
 import datetime
 import math
 import struct
 import warnings
-
-try:
-    from collections.abc import Mapping
-except ImportError:
-    from collections import Mapping
+from collections.abc import Mapping
 
 import numpy
 
@@ -48,7 +43,7 @@ _dtype_to_char = {
 }
 
 
-class Tree(object):
+class Tree:
     """
     Writes a TTree, including all TBranches, TLeaves, and (upon ``extend``) TBaskets.
 
@@ -121,7 +116,7 @@ class Tree(object):
 
             else:
                 try:
-                    if type(branch_type).__module__.startswith("awkward."):
+                    if uproot._util.from_module(branch_type, "awkward"):
                         raise TypeError
                     if (
                         uproot._util.isstr(branch_type)
@@ -133,12 +128,10 @@ class Tree(object):
                 except TypeError:
                     try:
                         awkward = uproot.extras.awkward()
-                    except ImportError as err:
+                    except ModuleNotFoundError as err:
                         raise TypeError(
-                            "not a NumPy dtype and 'awkward' cannot be imported: {0}\n\n{1}".format(
-                                repr(branch_type), str(err)
-                            )
-                        )
+                            f"not a NumPy dtype and 'awkward' cannot be imported: {branch_type!r}"
+                        ) from err
                     if isinstance(branch_type, awkward.types.Type):
                         branch_datashape = branch_type
                     else:
@@ -146,9 +139,7 @@ class Tree(object):
                             branch_datashape = awkward.types.from_datashape(branch_type)
                         except Exception:
                             raise TypeError(
-                                "not a NumPy dtype or an Awkward datashape: {0}".format(
-                                    repr(branch_type)
-                                )
+                                f"not a NumPy dtype or an Awkward datashape: {branch_type!r}"
                             )
                     # checking by class name to be Awkward v1/v2 insensitive
                     if type(branch_datashape).__name__ == "ArrayType":
@@ -175,7 +166,7 @@ class Tree(object):
                             dtype = numpy.dtype(content)
                         except Exception:
                             raise TypeError(
-                                "values of a dict must be NumPy types\n\n    key {0} has type {1}".format(
+                                "values of a dict must be NumPy types\n\n    key {} has type {}".format(
                                     repr(key), repr(content)
                                 )
                             )
@@ -242,7 +233,7 @@ class Tree(object):
                                 dtype = self._branch_ak_to_np(cont)
                                 if dtype is None:
                                     raise TypeError(
-                                        "fields of a record must be NumPy types, though the record itself may be in a jagged array\n\n    field {0} has type {1}".format(
+                                        "fields of a record must be NumPy types, though the record itself may be in a jagged array\n\n    field {} has type {}".format(
                                             repr(key), str(cont)
                                         )
                                     )
@@ -260,7 +251,7 @@ class Tree(object):
                         dt = self._branch_ak_to_np(content)
                         if dt is None:
                             raise TypeError(
-                                "cannot write Awkward Array type to ROOT file:\n\n    {0}".format(
+                                "cannot write Awkward Array type to ROOT file:\n\n    {}".format(
                                     str(branch_datashape)
                                 )
                             )
@@ -292,7 +283,7 @@ class Tree(object):
                             dtype = self._branch_ak_to_np(content)
                             if dtype is None:
                                 raise TypeError(
-                                    "fields of a record must be NumPy types, though the record itself may be in a jagged array\n\n    field {0} has type {1}".format(
+                                    "fields of a record must be NumPy types, though the record itself may be in a jagged array\n\n    field {} has type {}".format(
                                         repr(key), str(content)
                                     )
                                 )
@@ -304,7 +295,7 @@ class Tree(object):
 
                 else:
                     raise TypeError(
-                        "cannot write Awkward Array type to ROOT file:\n\n    {0}".format(
+                        "cannot write Awkward Array type to ROOT file:\n\n    {}".format(
                             str(branch_datashape)
                         )
                     )
@@ -366,16 +357,14 @@ class Tree(object):
 
         letter = _dtype_to_char.get(branch_dtype)
         if letter is None:
-            raise TypeError(
-                "cannot write NumPy dtype {0} in TTree".format(branch_dtype)
-            )
+            raise TypeError(f"cannot write NumPy dtype {branch_dtype} in TTree")
 
         if branch_shape == ():
             dims = ""
         else:
             dims = "".join("[" + str(x) + "]" for x in branch_shape)
 
-        title = "{0}{1}/{2}".format(branch_name, dims, letter)
+        title = f"{branch_name}{dims}/{letter}"
 
         return {
             "fName": branch_name,
@@ -412,7 +401,7 @@ class Tree(object):
         }
 
     def __repr__(self):
-        return "{0}({1}, {2}, {3}, {4}, {5}, {6}, {7})".format(
+        return "{}({}, {}, {}, {}, {}, {}, {})".format(
             type(self).__name__,
             self._directory,
             self._name,
@@ -487,6 +476,9 @@ class Tree(object):
             )
 
             for datum in self._branch_data:
+                if datum["kind"] == "record":
+                    continue
+
                 fBasketBytes = datum["fBasketBytes"]
                 fBasketEntry = datum["fBasketEntry"]
                 fBasketSeek = datum["fBasketSeek"]
@@ -517,23 +509,21 @@ class Tree(object):
             sink.flush()
 
         provided = None
-        module_name = type(data).__module__
 
-        if module_name == "pandas" or module_name.startswith("pandas."):
+        if uproot._util.from_module(data, "pandas"):
             import pandas
 
             if isinstance(data, pandas.DataFrame) and data.index.is_numeric():
                 provided = dataframe_to_dict(data)
 
-        if module_name == "awkward" or module_name.startswith("awkward."):
+        if uproot._util.from_module(data, "awkward"):
             try:
                 awkward = uproot.extras.awkward()
-            except ImportError as err:
+            except ModuleNotFoundError as err:
                 raise TypeError(
-                    "an Awkward Array was provided, but 'awkward' cannot be imported: {0}\n\n{1}".format(
-                        repr(data), str(err)
-                    )
-                )
+                    f"an Awkward Array was provided, but 'awkward' cannot be imported: {data!r}"
+                ) from err
+
             if isinstance(data, awkward.Array):
                 if data.ndim > 1 and not data.layout.purelist_isregular:
                     provided = {
@@ -559,12 +549,9 @@ class Tree(object):
 
             provided = {}
             for k, v in data.items():
-                module_name = type(v).__module__
-                if not (
-                    module_name == "pandas" or module_name.startswith("pandas.")
-                ) and not (
-                    module_name == "awkward" or module_name.startswith("awkward.")
-                ):
+                if not uproot._util.from_module(
+                    v, "pandas"
+                ) and not uproot._util.from_module(v, "awkward"):
                     if not hasattr(v, "dtype") and not isinstance(v, Mapping):
                         try:
                             with warnings.catch_warnings():
@@ -577,35 +564,28 @@ class Tree(object):
                         except (numpy.VisibleDeprecationWarning, Exception):
                             try:
                                 awkward = uproot.extras.awkward()
-                            except ImportError as err:
+                            except ModuleNotFoundError as err:
                                 raise TypeError(
-                                    "NumPy dtype would be dtype('O'), so we won't use NumPy, but 'awkward' cannot be imported: {0}: {1}\n\n{2}".format(
-                                        k, type(v), str(err)
-                                    )
-                                )
+                                    f"NumPy dtype would be dtype('O'), so we won't use NumPy, but 'awkward' cannot be imported: {k}: {type(v)}"
+                                ) from err
                             v = awkward.from_iter(v)
 
                     if getattr(v, "dtype", None) == numpy.dtype("O"):
                         try:
                             awkward = uproot.extras.awkward()
-                        except ImportError as err:
+                        except ModuleNotFoundError as err:
                             raise TypeError(
-                                "NumPy dtype is dtype('O'), so we won't use NumPy, but 'awkward' cannot be imported: {0}: {1}\n\n{2}".format(
-                                    k, type(v), str(err)
-                                )
-                            )
+                                f"NumPy dtype is dtype('O'), so we won't use NumPy, but 'awkward' cannot be imported: {k}: {type(v)}"
+                            ) from err
                         v = awkward.from_iter(v)
 
-                module_name = type(v).__module__
-                if module_name == "awkward" or module_name.startswith("awkward."):
+                if uproot._util.from_module(v, "awkward"):
                     try:
                         awkward = uproot.extras.awkward()
-                    except ImportError as err:
+                    except ModuleNotFoundError as err:
                         raise TypeError(
-                            "an Awkward Array was provided, but 'awkward' cannot be imported: {0}: {1}\n\n{2}".format(
-                                k, type(v), str(err)
-                            )
-                        )
+                            f"an Awkward Array was provided, but 'awkward' cannot be imported: {k}: {type(v)}"
+                        ) from err
                     if (
                         isinstance(v, awkward.Array)
                         and v.ndim > 1
@@ -616,7 +596,7 @@ class Tree(object):
                         if kk in provided:
                             if not numpy.array_equal(vv, provided[kk]):
                                 raise ValueError(
-                                    "branch {0} provided both as an explicit array and generated as a counter, and they disagree".format(
+                                    "branch {} provided both as an explicit array and generated as a counter, and they disagree".format(
                                         repr(kk)
                                     )
                                 )
@@ -625,7 +605,7 @@ class Tree(object):
                 if k in provided:
                     if not numpy.array_equal(v, provided[k]):
                         raise ValueError(
-                            "branch {0} provided both as an explicit array and generated as a counter, and they disagree".format(
+                            "branch {} provided both as an explicit array and generated as a counter, and they disagree".format(
                                 repr(kk)
                             )
                         )
@@ -637,8 +617,7 @@ class Tree(object):
                 if datum["name"] in provided:
                     recordarray = provided.pop(datum["name"])
 
-                    module_name = type(recordarray).__module__
-                    if module_name == "pandas" or module_name.startswith("pandas."):
+                    if uproot._util.from_module(recordarray, "pandas"):
                         import pandas
 
                         if isinstance(recordarray, pandas.DataFrame):
@@ -660,7 +639,7 @@ class Tree(object):
 
                 else:
                     raise ValueError(
-                        "'extend' must be given an array for every branch; missing {0}".format(
+                        "'extend' must be given an array for every branch; missing {}".format(
                             repr(datum["name"])
                         )
                     )
@@ -670,14 +649,14 @@ class Tree(object):
                     actual_branches[datum["fName"]] = provided.pop(datum["fName"])
                 else:
                     raise ValueError(
-                        "'extend' must be given an array for every branch; missing {0}".format(
+                        "'extend' must be given an array for every branch; missing {}".format(
                             repr(datum["fName"])
                         )
                     )
 
         if len(provided) != 0:
             raise ValueError(
-                "'extend' was given data that do not correspond to any branch: {0}".format(
+                "'extend' was given data that do not correspond to any branch: {}".format(
                     ", ".join(repr(x) for x in provided)
                 )
             )
@@ -689,7 +668,7 @@ class Tree(object):
                 num_entries = len(branch_array)
             elif num_entries != len(branch_array):
                 raise ValueError(
-                    "'extend' must fill every branch with the same number of entries; {0} has {1} entries".format(
+                    "'extend' must fill every branch with the same number of entries; {} has {} entries".format(
                         repr(branch_name),
                         len(branch_array),
                     )
@@ -703,7 +682,7 @@ class Tree(object):
                 big_endian = numpy.asarray(branch_array, dtype=datum["dtype"])
                 if big_endian.shape != (len(branch_array),) + datum["shape"]:
                     raise ValueError(
-                        "'extend' must fill branches with a consistent shape: has {0}, trying to fill with {1}".format(
+                        "'extend' must fill branches with a consistent shape: has {}, trying to fill with {}".format(
                             datum["shape"],
                             big_endian.shape[1:],
                         )
@@ -718,12 +697,10 @@ class Tree(object):
             else:
                 try:
                     awkward = uproot.extras.awkward()
-                except ImportError as err:
+                except ModuleNotFoundError as err:
                     raise TypeError(
-                        "a jagged array was provided (possibly as an iterable), but 'awkward' cannot be imported: {0}: {1}\n\n{2}".format(
-                            branch_name, repr(branch_array), str(err)
-                        )
-                    )
+                        f"a jagged array was provided (possibly as an iterable), but 'awkward' cannot be imported: {branch_name}: {branch_array!r}"
+                    ) from err
                 layout = branch_array.layout
                 while not isinstance(
                     layout,
@@ -805,7 +782,7 @@ class Tree(object):
 
                 if shape[1:] != datum["shape"]:
                     raise ValueError(
-                        "'extend' must fill branches with a consistent shape: has {0}, trying to fill with {1}".format(
+                        "'extend' must fill branches with a consistent shape: has {}, trying to fill with {}".format(
                             datum["shape"],
                             shape[1:],
                         )
